#include <cstdlib>
#include "object.h"
#include "file/format.h"
#include "classmap.h"
#include "st_common.h"

extern graphicsLib graphLib;
extern std::string FILEPATH;

#include "timerlib.h"
extern timerLib timer;

#include "graphicslib.h"
extern graphicsLib graphLib;

#include "soundlib.h"
extern soundLib soundManager;


std::map<unsigned int, graphicsLib_gSurface> object::objects_sprite_list;

extern CURRENT_FILE_FORMAT::file_game game_data;

#define GRAVITY_SPEED 4

#include "game.h"
extern game gameControl;

#define DISAPPEARNING_VISIBLE_TIME 1500
#define DISAPPEARNING_HIDDEN_TIME 2000

// constructor for game_object


// constructor for map_object
object::object(int set_id, classMap *set_map, st_position map_pos) : _finished(false), _state(0), _duration(0), _timer_limit(0), _started(false), _animation_finished(false), _animation_reversed(false), _hidden(false)
{
	map = set_map;
	_id = set_id;
	name = std::string(game_data.objects[_id].name);
	graphic_filename = game_data.objects[_id].graphic_filename;
	type = game_data.objects[_id].type;
	obj_timer = game_data.objects[_id].timer;
	speed = game_data.objects[_id].speed;
	limit = game_data.objects[_id].limit;
	direction = 0;
	distance = 0;
	framesize_w = game_data.objects[_id].size.width;
	framesize_h = game_data.objects[_id].size.height;
	frame = 0;
	start_point.x = map_pos.x*TILESIZE;
	start_point.y = map_pos.y*TILESIZE;
	position.x = map_pos.x*TILESIZE;
	position.y = map_pos.y*TILESIZE;
	//std::cout << ">>> NEW map_pos.x: " << map_pos.x << ", map_pos.y: " << map_pos.y << ", obj.x: " << position.x << ", y: " << position.y << std::endl;
	//std::string complete_filename(FILEPATH + graphic_filename);
	if (limit == 0 && (type == OBJ_MOVING_PLATFORM_LEFTRIGHT || type == OBJ_MOVING_PLATFORM_UPDOWN)) {
		limit = 48;
    } else if (type == OBJ_DISAPPEARING_BLOCK) {
        _hidden = true;
    }
	//std::cout << ">>>>> RESET COMMAND UP" << std::endl;
	_command_up = false;
	_command_down = false;
    add_graphic();
}

void object::reset()
{
    _finished = false;
    _state = 0;
    _duration = 0;
    _timer_limit = 0;
    _started = false;
    _animation_finished = false;
    _animation_reversed = false;
    _hidden = false;
    direction = 0;
    distance = 0;
    position.x = start_point.x;
    position.y = start_point.y;
    _started = false;
    if (limit == 0 && (type == OBJ_MOVING_PLATFORM_LEFTRIGHT || type == OBJ_MOVING_PLATFORM_UPDOWN)) {
        limit = 48;
    } else if (type == OBJ_DISAPPEARING_BLOCK) {
        _hidden = true;
    }
    _command_up = false;
    _command_down = false;

}


// ********************************************************************************************** //
// adds an entry into object_graphics_list map, if needed                                      //
// ********************************************************************************************** //
void object::add_graphic()
{
	std::map<unsigned int, graphicsLib_gSurface>::iterator it;
	graphicsLib_gSurface temp_sprite;

	it = objects_sprite_list.find(_id);
	if (it == objects_sprite_list.end()) { // there is no graphic with this key yet, add it
		std::string complete_filename(FILEPATH + "data/images/sprites/objects/" + graphic_filename);
		graphLib.surfaceFromFile(complete_filename, &temp_sprite);
		objects_sprite_list.insert(std::pair<unsigned int, graphicsLib_gSurface>(_id, temp_sprite));
        //std::cout << "2. object::object - adding graphic entry for object[" << _id << "] - '" << name << "', list.size: " << objects_sprite_list.size() << std::endl;
	}
}

void object::gravity()
{
	// non-falling object types
    if (type == OBJ_MOVING_PLATFORM_UPDOWN || type == OBJ_MOVING_PLATFORM_LEFTRIGHT || type == OBJ_DISAPPEARING_BLOCK || type == OBJ_FALL_PLATFORM || type == OBJ_ITEM_FLY || type == OBJ_FLY_PLATFORM || type == OBJ_ACTIVE_DISAPPEARING_BLOCK) {
		return;
	}
	for (int i=GRAVITY_SPEED; i>0; i--) {
		if (test_change_position(0, i) == true) {
			position.y += i;
			check_player_move(0, i);
			break;
		}
	}
	if (position.y > RES_H) {
		_finished = true;
	}
}

bool object::test_change_position(short xinc, short yinc)
{
	if (map == NULL) {
		return false;
	}
	if (xinc == 0 && yinc == 0) {
		return true;
	}

    if (yinc > 0 && position.y > RES_H) { // check if item felt out of screen
        if (position.y < RES_H+TILESIZE) {
            _finished = true;
            return false;
        } else {
            return true;
        }

    }

	short p1 = map->getMapPointLock(st_position((position.x+2+xinc)/TILESIZE, (position.y+yinc+framesize_h-2)/TILESIZE));
	short p2 = map->getMapPointLock(st_position((position.x+framesize_w-2+xinc)/TILESIZE, (position.y+yinc+framesize_h-2)/TILESIZE));
	//std::cout << "object::test_change_position - p1: " << p1 << ", p2: " << p2 << std::endl;
	if ((p1 == TERRAIN_UNBLOCKED ||  p1 == TERRAIN_WATER) && (p2 ==TERRAIN_UNBLOCKED ||  p2 == TERRAIN_WATER)) {
		return true;
	}
	return false;
}

void object::check_player_move(int xinc, int yinc)
{
	//std::cout << "object::check_player_move::START - p.platform: " << gameControl.get_player_platform() << ", this: " << this << std::endl;
	if (xinc == 0 && yinc == 0) {
		//std::cout << "object::check_player_move::LEAVE" << std::endl;
		return;
	}
	if (yinc < 0 && position.y+framesize_h < 0) {
		return;
	}
	if (yinc > 0 && position.y > RES_H) {
		return;
    }
	if (gameControl.get_player_platform() == this) {
		//std::cout << "************* object::check_player_move - MOVE xinc: " << xinc << ", yinc: " << yinc << " **************" << std::endl;
		gameControl.change_player_position(xinc, yinc, 0);
		//std::cout << ">> check_player_move - new p_pos yinc: " << yinc << ", y: " << p_pos.y << std::endl;
    }
}

void object::set_colision_mode(colision_modes colision_mode)
{
    _colision_mode = colision_mode;
}

colision_modes object::get_colision_mode()
{
    return _colision_mode;
}



// ********************************************************************************************** //
// adds an entry into object_graphics_list map, if needed                                      //
// ********************************************************************************************** //
void object::show()
{
	st_rectangle graphic_origin;
	st_position graphic_destiny;

	if (map == NULL) {
		return;
	}

	if (objects_sprite_list.find(_id) == objects_sprite_list.end()) {
		//std::cout << "object::show - could not find graphic for object with id " << _id << std::endl;
		return;
	}

	//printf("********* check draw object, x: %d, y: %d\n", position.x, position.y);
    if (_hidden == true) {
		//printf("********* show_object_sprites - hidden object\n");
		return;
	}

	int max_frames = ((objects_sprite_list.find(_id)->second).width/framesize_w)-1;

	// checks if the Object is near the screen to show it
	if (position.x+16 >= abs((float)map->getMapScrolling().x) && position.x-16 <= abs((float)map->getMapScrolling().x)+RES_W) {
		// animation

		//std::cout << "object::show - frame_duration: " << game_data.objects[_id].frame_duration << std::endl;

		if ((game_data.objects[_id].animation_auto_start == true || (game_data.objects[_id].animation_auto_start == false && _started == true)) && framesize_w * 2 <= (objects_sprite_list.find(_id)->second).width)  { // have at least two frames
			graphic_origin.x = frame * framesize_w;
			if (obj_timer < timer.getTimer()) {
				if (_animation_finished == false) { //
					if (_animation_reversed == false) {
						frame++;
					} else {
						frame--;
					}
				}
				obj_timer = timer.getTimer()+game_data.objects[_id].frame_duration;
			}

			if (frame <= 0) {
				if (_animation_reversed == true) {
					_animation_finished = false;
					_animation_reversed = false;
					_started = false;
				}
				frame = 0;
			}
			if	(_animation_reversed == false && frame > max_frames) {
				if (game_data.objects[_id].animation_loop == false) { // if animation loop is set to false, set this to show always the last frame
					if (game_data.objects[_id].animation_reverse == false) { // don't need to reverse animation, finish it
						_animation_finished = true;
						frame = (objects_sprite_list.find(_id)->second).width/framesize_w-1;
					} else {
						_animation_reversed = true; // start animation reverse process
						frame--;
					}
				} else {
					frame = 0;
				}
			}
		} else {
			graphic_origin.x = 0;
		}

		//std::cout << "object::show - frame_n: " << frame << ", _animation_reversed: " << _animation_reversed << ", max_frames: " << max_frames << std::endl;


		// direction
		//printf(">> sprite->h: %d, framesize_h*2: %d <<\n", sprite->h, (framesize_h*2));
		if (framesize_h*2 <= (objects_sprite_list.find(_id)->second).height)  {
			//std::cout << ">>>> object height is enought for direction-right";
			if (direction != ANIM_DIRECTION_RIGHT) {
				graphic_origin.y = 0;
			} else {
				graphic_origin.y = framesize_h;
			}
		} else {
			//std::cout << ">>>> NOT object image-height(" << (objects_sprite_list.find(name)->second).height << ") is NOT enought for direction-right. framesize_h: " << framesize_h << std::endl;
			graphic_origin.y = 0;
		}


		graphic_origin.w = framesize_w;
		graphic_origin.h = framesize_h;

		// parte que vai ser colada
		graphic_destiny.x = position.x - map->getMapScrolling().x;
		graphic_destiny.y = position.y + map->getMapScrolling().y;
		//std::cout << "searching for graphic '" << name << "'" << std::endl;
		//std::cout << "object::show - map->getMapScrolling().x: " << map->getMapScrolling().x << ", map->getMapScrolling().y: " << map->getMapScrolling().y << ", position.y: " << position.y << ", graphic_destiny.x: " << graphic_destiny.x << ", graphic_destiny.y: " << graphic_destiny.y << std::endl;

		std::map<unsigned int, graphicsLib_gSurface>::iterator it;
		it = objects_sprite_list.find(_id);
		if (it != objects_sprite_list.end()) { // there is no graphic with this key yet, add it
			graphLib.copyArea(st_rectangle(graphic_origin.x, graphic_origin.y, graphic_origin.w, graphic_origin.h), st_position(graphic_destiny.x, graphic_destiny.y), &it->second, &graphLib.gameScreen);
            // disappearning block has a shadow under it
            if (type == OBJ_DISAPPEARING_BLOCK)             {
                graphLib.clear_area(graphic_destiny.x, graphic_destiny.y+framesize_h, framesize_w, 6, 0, 0, 0);
            }
		}



	// remove item that is out of vision
	} else if (type == OBJ_ITEM_FLY || type == OBJ_ITEM_JUMP) {
		printf(">> remove item <<\n");
		_finished = true;
	}
}

void object::move()
{
    //std::cout << "name: " << name << ", _duration: " << _duration << ", time-limit: " << _timer_limit << ", timer: " << timer.getTimer() << std::endl;
	if (_duration > 0 && timer.getTimer() > _timer_limit) {
		_finished = true;
		return;
	}
	//std::cout << "object::move::START" << std::endl;
	if (type == OBJ_MOVING_PLATFORM_LEFTRIGHT) {
		if (distance > limit) {
			invert_direction_x();
			distance = 0;
		}
		int xinc=0;
		if (direction == ANIM_DIRECTION_LEFT) {
			xinc = -speed;
		} else {
			xinc = speed;
		}
		bool can_move = test_change_position(xinc, 0);
		//std::cout << "object::move::OBJ_MOVING_PLATFORM_LEFTRIGHT - xinc: " << xinc << ", can_move: " << can_move << ", distance: " << distance << ", limit: " << limit << std::endl;
		if (can_move) {
			position.x += xinc;
			check_player_move(xinc, 0);
			distance += abs((float)xinc);
		} else {
			invert_direction_x();
			distance = 0;
		}
	} else if (type == OBJ_MOVING_PLATFORM_UPDOWN) {
		if (distance > limit) {
			//std::cout << "OBJ_MOVING_PLATFORM_UPDOWN - limit reached - distance: " << distance << ", limit: " << limit << ", invert" << std::endl;
			invert_direction_y();
			distance = 0;
		}
		int yinc=0;
		if (direction == ANIM_DIRECTION_UP) {
			yinc = -speed;
		} else {
			yinc = speed;
		}
		bool can_move = test_change_position(0, yinc);
		//std::cout << "object::move::OBJ_MOVING_PLATFORM_LEFTRIGHT - xinc: " << xinc << ", can_move: " << can_move << ", distance: " << distance << ", limit: " << limit << std::endl;
		if (can_move) {
			position.y += yinc;
			check_player_move(0, yinc);
			distance += abs((float)yinc);
		} else {
			std::cout << "1. OBJ_MOVING_PLATFORM_UPDOWN - colision, invert_direction_y - direction: " << direction << std::endl;
			invert_direction_y();
			std::cout << "2. OBJ_MOVING_PLATFORM_UPDOWN - colision, invert_direction_y - direction: " << direction << std::endl;
			distance = 0;
		}
	} else if (type == OBJ_FALL_PLATFORM) {
		if (_state == OBJ_STATE_MOVE) { // falling state
			int yinc = speed;
			bool can_move = test_change_position(0, yinc);
			if (can_move) {
				position.y += yinc;
				check_player_move(0, yinc);
				distance += abs((float)yinc);
			} else {
				_state = OBJ_STATE_RETURN;
			}
		} else if (_state == OBJ_STATE_RETURN) { // returning state
			int yinc = -speed/2;
			if (yinc == 0) {
				yinc = -1;
			}
			position.y += yinc;
			check_player_move(0, yinc);
			distance -= abs((float)yinc);
			if (distance == 0) {
				_state = OBJ_STATE_STAND;
			}
		}
		// check out of screen
		if (_state == 1 && position.y > RES_H+TILESIZE*3) {
			_state = OBJ_STATE_RETURN;
		}
	} else if (type == OBJ_FLY_PLATFORM) {
		if (_state == OBJ_STATE_MOVE) {
			int yinc = -speed;
			position.y += yinc;
			check_player_move(0, yinc);
			distance += abs((float)yinc);
		} else if (_state == OBJ_STATE_RETURN) {
			int yinc = speed/2;
			if (yinc == 0) {
				yinc = 1;
			}
			position.y += yinc;
			check_player_move(0, yinc);
			distance -= abs((float)yinc);
			if (distance == 0) {
				_state = 0;
			}
		}
		// check out of screen
		if (_state == 1 && position.y+framesize_h < -TILESIZE*3) {
			_state = OBJ_STATE_RETURN;
		}
	} else if (type == OBJ_ITEM_FLY && _started == true) {
		if (speed < 1) {
			speed = 3;
		}
		int xinc = speed;
		int yinc = 0;
		if (direction == ANIM_DIRECTION_LEFT) {
			xinc = -speed;
		}
		if (_command_up == true) {
			yinc = -speed;
		}
		if (_command_down == true) {
			yinc = speed;
		}
		position.x += xinc;
		position.y += yinc;
		std::cout << "OBJ FLY STARTED - move - xinc: " << xinc << ", position.x: " << position.x << std::endl;
		check_player_move(xinc, yinc); // @TODO - player can move up/down
		distance += abs((float)xinc);
		_command_up = false;
		_command_down = false;
    } else if (type == OBJ_DISAPPEARING_BLOCK) { // 1500 is visible time and 2000 is hidden time. initial delay is defined by obj_timer
        if (_timer_limit < timer.getTimer()) {
            if (_timer_limit == 0) {
                _timer_limit = timer.getTimer() + obj_timer;
            } else {
                //std::cout << "OBJ_DISAPPEARING_BLOCK - obj_timer: " << obj_timer << ", limit: " << limit << std::endl;
                int timer_add = DISAPPEARNING_HIDDEN_TIME;
                if (_hidden == false) {
                    timer_add = DISAPPEARNING_VISIBLE_TIME;
                }
                _timer_limit = timer.getTimer() + timer_add;
                _hidden = !_hidden;
                if (_hidden == false && is_on_screen() == true) {
                    soundManager.play_sfx(SFX_DISAPPEARING_BLOCK);
                }
            }
        }
    } else if (type == OBJ_ACTIVE_DISAPPEARING_BLOCK) {
        //std::cout << "OBJ_ACTIVE_DISAPPEARING_BLOCK - MOVE - _started: " << _started << ", _timer_limit: " << _timer_limit << ", timer.now: " << timer.getTimer() << std::endl;
        if (_started == true && _timer_limit < timer.getTimer()) {
            if (_hidden == false) { // turn hidden
                //std::cout << "OBJ_ACTIVE_DISAPPEARING_BLOCK - HIDE" << std::endl;
                _hidden = true;
                _timer_limit = timer.getTimer() + DISAPPEARNING_HIDDEN_TIME;
            } else { // return to visible state
                //std::cout << "OBJ_ACTIVE_DISAPPEARING_BLOCK - SHOW" << std::endl;
                _started = false;
                _hidden = false;
            }
        }
	}
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void object::execute()
{
	move();
	gravity();
	show();
	/// @TODO: finish items
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
st_position object::get_position()
{
	return position;
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
st_size object::get_size()
{
	return st_size(framesize_w, framesize_h);
}


// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
short int object::get_type()
{
	return type;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
int object::get_direction()
{
	return direction;
}

// ********************************************************************************************** //
//                                                                                                //
// ********************************************************************************************** //
void object::set_direction(int new_dir)
{
	direction = new_dir;
}

int object::get_distance()
{
	return distance;
}



void object::set_distance(int new_dist)
{
	distance = new_dist;
}


void object::set_timer(int new_timer)
{
	obj_timer = new_timer;
}

unsigned int object::get_timer()
{
	return obj_timer;
}

bool object::finished()
{
    //std::cout << ">> object::finished: " << _finished << std::endl;
	return _finished;
}

void object::set_finished(bool is_finished)
{
    std::cout << ">> object::set_finished: " << is_finished << std::endl;
	_finished = is_finished;
}

void object::invert_direction_x()
{
	if (direction == ANIM_DIRECTION_LEFT) {
		//std::cout << "invert left->right" << std::endl;
		direction = ANIM_DIRECTION_RIGHT;
	} else {
		//std::cout << "invert right->left" << std::endl;
		direction = ANIM_DIRECTION_LEFT;
	}
}

void object::invert_direction_y() {
	if (direction == ANIM_DIRECTION_UP) {
		//std::cout << "invert up->down" << std::endl;
		direction = ANIM_DIRECTION_DOWN;
	} else {
		//std::cout << "invert down->up" << std::endl;
		direction = ANIM_DIRECTION_UP;
	}
}

void object::set_state(short obj_state)
{
	_state = obj_state;
	//std::cout << "********* object::set_state::state: " << _state << std::endl;
}

short object::get_state()
{
	return _state;
}

void object::set_duration(int duration)
{
	_duration = duration;
	_timer_limit = timer.getTimer() + duration;
}

void object::start()
{
	_started = true;
    if (type == OBJ_ACTIVE_DISAPPEARING_BLOCK) {
        //std::cout << "OBJ_ACTIVE_DISAPPEARING_BLOCK - STARTED - obj_timer: " << obj_timer << std::endl;
        _timer_limit = timer.getTimer() + obj_timer;
    }
}

void object::command_up()
{
	_command_up = true;
	std::cout << ">>>>> SET COMMAND UP" << std::endl;
}

void object::command_down()
{
	_command_down = true;
}

std::string object::get_name()
{
    return name;
}

bool object::is_hidden()
{
    //std::cout << "object::is_hidden - _hidden: " << _hidden << std::endl;
    return _hidden;
}

bool object::is_started()
{
    return _started;
}

bool object::is_on_screen()
{
    st_position scroll = map->getMapScrolling();
    // entre scroll.x-RES_W/2 e scroll.x+RES_W+RES_W/2

    //if (realPosition.x < -TILESIZE*2 || realPosition.x > (RES_W+TILESIZE*2)) {
    if (abs((float)position.x) > scroll.x-RES_W/2 && abs((float)position.x) < scroll.x+RES_W*1.5) {
        return true;
    }
    return false;
}

int object::get_duration()
{
    return _duration;
}

